/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.common.binaryfile;

import java.io.InputStream;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.function.Function;

import io.grpc.stub.CallStreamObserver;
import io.grpc.stub.StreamObserver;
import lombok.AccessLevel;
import lombok.NoArgsConstructor;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;

@Log4j2
@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class GrpcFileUploadUtils {

	/**
	 * @param <Q> Request Type
	 * @param <S> Response Type
	 * @deprecated use {@link #createStreamExclusiveFileSender(BiFunction, InputStream, Function)} instead
	 */
	@Deprecated(since = "4.13.0")
	public static <Q, S> FileSender<Q, S> createSender(BiFunction<byte[], Integer, Q> chunkBuilder, InputStream inputStream,
			Function<StreamObserver<S>, CallStreamObserver<Q>> reqObserverBuilder) {
		return new FileSender<>(chunkBuilder, inputStream, reqObserverBuilder);
	}

	/**
	 * @param <Q> Request Type
	 * @param <S> Response Type
	 */
	public static <Q, S> StreamingFileSender<Q, S> createStreamExclusiveFileSender(BiFunction<byte[], Integer, Q> chunkBuilder, InputStream inputStream,
			Function<StreamObserver<S>, CallStreamObserver<Q>> reqObserverBuilder) {
		return new StreamExclusiveFileSender<>(chunkBuilder, inputStream, reqObserverBuilder);
	}

	/**
	 * @param <Q> Request Type
	 * @param <S> Response Type
	 */
	public static <Q, S> StreamingFileSender<Q, S> createStreamSharingSender(BiFunction<byte[], Integer, Q> chunkBuilder, InputStream inputStream,
			CallStreamObserver<Q> requestObserver, Consumer<Runnable> onReadyHandlerRegistrar) {
		return new StreamSharingFileSender<>(chunkBuilder, inputStream, requestObserver, onReadyHandlerRegistrar);
	}

	@Deprecated(since = "4.13.0")
	public static class FileSender<Q, S> extends StreamExclusiveFileSender<Q, S> {

		FileSender(BiFunction<byte[], Integer, Q> chunkBuilder, InputStream inputStream,
				Function<StreamObserver<S>, CallStreamObserver<Q>> reqObserverBuilder) {
			super(chunkBuilder, inputStream, reqObserverBuilder);
		}

		@Override
		public FileSender<Q, S> send() {
			super.send();
			return this;
		}

		@Override
		public FileSender<Q, S> withMetaData(@NonNull Q metaData) {
			super.withMetaData(metaData);
			return this;
		}
	}

}